/**
File:    ADF_U1.h
  ----------------------------------------------------------------------
                        BOEING
  ----------------------------------------------------------------------
    Project: CGNS
    Author:  Wayne Jones 655-5162    wrj9493@lippy.ca.boeng.com
    Date:    7/8/96
    Purpose: Provide prototype declarations for the ADF-Level 1 Utility
             routines.
  ----------------------------------------------------------------------
  ----------------------------------------------------------------------

**/

#ifndef ADF_U1_INCLUDE
#define ADF_U1_INCLUDE

/* The following is taken out of ADF.h */
/***********************************************************************
    Defines:  These defines are used within the ADF core routines
    to specify the size in bytes of varoius items.
   Caution:  Simply changing a define here may not correctly adjust the
    ADF core code.  These sizes are provided for reference only!
***********************************************************************/
#define ADF_DATA_TYPE_LENGTH        32
#define ADF_DATE_LENGTH             32
#define ADF_FILENAME_LENGTH       1024
#define ADF_FORMAT_LENGTH           20
#define ADF_LABEL_LENGTH            32
#define ADF_MAXIMUM_LINK_DEPTH     100
#define ADF_MAX_DIMENSIONS          12
#define ADF_MAX_ERROR_STR_LENGTH    80
#define ADF_MAX_LINK_DATA_SIZE    4096
#define ADF_NAME_LENGTH             32
#define ADF_STATUS_LENGTH           32
#define ADF_VERSION_LENGTH          32


/***********************************************************************
    Prototypes for Interface Routines
***********************************************************************/

#if defined (__cplusplus)
    extern "C" {
#endif

extern    void    AU1_Count_Nodes(
            const double ID,
            const char *name,
            const char *label,
            const int depth,
            int *num_nodes,
            int *error_return ) ;

extern    void    AU1_Return_Node_IDs(
            const double ID,
            const char *name,
            const char *label,
            const int istart,
            const int ilen,
            const int depth,
            int *ilen_ret,
            double *node_id,
            int *error_return ) ;

extern    void    AU1_Count_Ancestors(
            const double ID,
            const char *name,
            const char *label,
            int *num_ancestors,
            int *error_return );

extern    void    AU1_Return_Ancestor_IDs(
            const double ID,
            const char *name,
            const char *label,
            const int istart,
            const int ilen,
            int *ilen_ret,
            double *node_id,
            int *error_return );

extern    void    AU1_Error_Message(
            const int error_return_input,
            char *error_string );

#if defined (__cplusplus)
    }
#endif
#endif
