c
c   CFD Utility Interface Subroutines:
c
c   Naming convention is to use AUixxxx for ADF_Utilities_ith 
c   library with routine name xxxx.  The CFD Utilities is the
c   2nd ADF Utilities library.
c
c
c   X = done  s=started  w=working (more to add)
c      status of routines:                       | .f  | .c  |     |
c      status of routines:                   rel |entry| glue|testd|
c                                                |  -  |  -  |     |
c                                                +-----+-----+-----+
c   AU1CN     AU1_Count_Nodes                 1  |  X  |  X  |     |
c   AU1RNID   AU1_Return_Node_IDs             1  |  X  |  X  |     |
c   AU1CAN    AU1_Count_Ancestors             1  |  X  |  X  |     |
c   AU1RAID   AU1_Return_Ancestor_IDs         1  |  X  |  X  |     |
c                                                +-----+-----+-----+
c   
c   Usage Conventions & Implementations:
c   This describes specific details about the interface routines:
c   Character variables:  
c   C: All input strings are to be NULL terminated.  All retuned string 
c   will be NULL terminated and will have any trailing blanks removed.
c   FORTRAN: Strings sizes will be determined using inherited length.  
c   Returned strings will be blank-filled to the specified length.
c   const: The term "const" used below indicates an argument which is 
c   read-only.
c   ierr: The error code returned from ADF routines is as follows:
c   -1 : No Error
c   Positive number : the ADF error code.  The routine 
c   indexing: All indexing is FORTRAN-like, that is, a starting index is
c   1 and the last index for N things is N.  This is used when specifying 
c   dimensions in ADF_Read_Data and ADF_Write_Data, the starting 
c   child in ADF_Children_Names.
c   

c   


c***********************************************************************
c   R1: Count the number of nodes having the given name and label.
c
c
c
c
c   f77: AU1CN(id,name,label,depth,num_nodes,ierr )
c   input:  real*8 id.
c   input:  character*(*) name.
c   input:  character*(*) label.
c   input:  integer depth.
c   output:  integer num_nodes.
c   output:  integer ierr.
c***********************************************************************
      subroutine AU1CN(id, name, label, depth, num_nodes, ierr )
      IMPLICIT NONE
      real*8 id
      character*(*) name
      character*(*) label
      integer depth
      integer num_nodes
      integer ierr

      call AU1CN2( id, name, LEN(name), label, LEN(label), depth, 
     $    num_nodes, ierr )
      return
      end


c***********************************************************************
c   R1: Return the node ids of nodes having the given name and label.
c
c
c
c
c   f77: AU1RNID(id,name,label,istart,ilen,depth,ilen_ret,node_id,ierr)
c   input:  real*8 id.
c   input:  character*(*) name.
c   input:  character*(*) label.
c   input:  integer istart.
c   input:  integer ilen.
c   input:  integer depth.
c   output:  integer ilen_ret.
c   output:  real*8 node_id.
c   output:  integer ierr.
c***********************************************************************
      subroutine AU1RNID(id, name, label, istart, ilen, depth, ilen_ret, 
     $    node_id, ierr )
      IMPLICIT NONE
      real*8 id
      character*(*) name
      character*(*) label
      integer istart
      integer ilen
      integer depth
      integer ilen_ret
      real*8  node_id
      integer ierr

      call AU1RNID2( id, name, LEN(name), label, LEN(label), istart,
     $    ilen, depth, ilen_ret, node_id, ierr )
      return
      end


c***********************************************************************
c   R1: Count the number of ancestors having the given name and label.
c
c
c
c
c   f77: AU1CAN(id,name,label,nancstr,ierr )
c   input:  real*8 id.
c   input:  character*(*) name.
c   input:  character*(*) label.
c   output:  integer nancstr.
c   output:  integer ierr.
c***********************************************************************
      subroutine AU1CAN(id, name, label, nancstr, ierr )
      IMPLICIT NONE
      real*8 id
      character*(*) name
      character*(*) label
      integer nancstr
      integer ierr

      call AU1CAN2( id, name, LEN(name), label, LEN(label), 
     $    nancstr, ierr )
      return
      end


c***********************************************************************
c   R1: Return the node ids of ancestors having the given name and 
c       label.
c
c
c
c
c   f77: AU1RAID(id,name,label,istart,ilen,ilen_ret,node_id,ierr)
c   input:  real*8 id.
c   input:  character*(*) name.
c   input:  character*(*) label.
c   input:  integer istart.
c   input:  integer ilen.
c   input:  integer depth.
c   output:  integer ilen_ret.
c   output:  real*8 node_id.
c   output:  integer ierr.
c***********************************************************************
      subroutine AU1RAID(id, name, label, istart, ilen, ilen_ret, 
     $    node_id, ierr )
      IMPLICIT NONE
      real*8 id
      character*(*) name
      character*(*) label
      integer istart
      integer ilen
      integer ilen_ret
      real*8  node_id
      integer ierr

      call AU1RAID2( id, name, LEN(name), label, LEN(label), istart,
     $    ilen, ilen_ret, node_id, ierr )
      return
      end

