/***********************************************************************
    CFD Utilities Core:
    Naming convention is to use AUixxxx for ADF_Utilities_ith 
    library with routine name xxxx.  

   Glue routines between the FORTRAN interface and the C interface.

***********************************************************************/

#ifdef    __sgi
#define    AU1CN2     au1cn2_
#define    AU1RNID2   au1rnid2_
#define    AU1CAN2    au1can2_
#define    AU1RAID2   au1raid2_
#endif

#include <stdio.h>
/*
#include "ADF.h"
#include "ADF_internals.h"
*/
#include "ADF_U1.h"
#include "ADF_U1_internals.h"

/***********************************************************************
AU1CN    Count_Nodes:
***********************************************************************/
void    AU1CN2(
                const double *rid,
                const char *name,
                const int *name_length,
                const char *label,
                const int *label_length,
                const int *depth,
                int *num_nodes,
                int *error_return )

{
  char    c_name [ ADF_NAME_LENGTH + 1 ] ;
  char    c_label[ ADF_NAME_LENGTH + 1 ] ;

  AU1I_string_2_C_string( name, MIN(ADF_NAME_LENGTH, *name_length), 
    c_name, error_return ) ;
  if( *error_return != NO_ERROR )
    return ;

  AU1I_string_2_C_string(label, MIN(ADF_NAME_LENGTH, *label_length), 
    c_label, error_return ) ;
  if( *error_return != NO_ERROR )
    return ;

  AU1_Count_Nodes( *rid, c_name, c_label, *depth, num_nodes, error_return ) ;
}

/***********************************************************************
AU1RNID  Return node ids:
***********************************************************************/
void    AU1RNID2(
                const double *rid,
                const char *name,
                const int *name_length,
                const char *label,
                const int *label_length,
                const int *istart,
                const int *ilen,
                const int *depth,
                int *ilen_ret,
                double *node_id,
                int *error_return )


{
  char    c_name [ ADF_NAME_LENGTH + 1 ] ;
  char    c_label[ ADF_NAME_LENGTH + 1 ] ;

  AU1I_string_2_C_string( name, MIN(ADF_NAME_LENGTH, *name_length), 
    c_name, error_return ) ;
  if( *error_return != NO_ERROR )
    return ;

  AU1I_string_2_C_string(label, MIN(ADF_NAME_LENGTH, *label_length), 
    c_label, error_return ) ;
  if( *error_return != NO_ERROR )
    return ;

  AU1_Return_Node_IDs( *rid, c_name, c_label, *istart, *ilen, *depth, ilen_ret, node_id, error_return ) ;

}


/***********************************************************************
AU1CAN    Count_Ancestors:
***********************************************************************/
void    AU1CAN2(
                const double *rid,
                const char *name,
                const int *name_length,
                const char *label,
                const int *label_length,
                int *num_nodes,
                int *error_return )

{
  char    c_name [ ADF_NAME_LENGTH + 1 ] ;
  char    c_label[ ADF_NAME_LENGTH + 1 ] ;

  AU1I_string_2_C_string( name, MIN(ADF_NAME_LENGTH, *name_length), 
    c_name, error_return ) ;
  if( *error_return != NO_ERROR )
    return ;

  AU1I_string_2_C_string(label, MIN(ADF_NAME_LENGTH, *label_length), 
    c_label, error_return ) ;
  if( *error_return != NO_ERROR )
    return ;

  AU1_Count_Ancestors( *rid, c_name, c_label, num_nodes, error_return ) ;
}

/***********************************************************************
AU1RANID  Return ancestors' ids:
***********************************************************************/
void    AU1RAID2(
                const double *rid,
                const char *name,
                const int *name_length,
                const char *label,
                const int *label_length,
                const int *istart,
                const int *ilen,
                int *ilen_ret,
                double *node_id,
                int *error_return )


{
  char    c_name [ ADF_NAME_LENGTH + 1 ] ;
  char    c_label[ ADF_NAME_LENGTH + 1 ] ;

  AU1I_string_2_C_string( name, MIN(ADF_NAME_LENGTH, *name_length), 
    c_name, error_return ) ;
  if( *error_return != NO_ERROR )
    return ;

  AU1I_string_2_C_string(label, MIN(ADF_NAME_LENGTH, *label_length), 
    c_label, error_return ) ;
  if( *error_return != NO_ERROR )
    return ;

  AU1_Return_Ancestor_IDs( *rid, c_name, c_label, *istart, *ilen, ilen_ret, node_id, error_return ) ;

}

