#include <stdlib.h>
#include <stdio.h>
#include <ctype.h>
#include <string.h>
#include "ADF.h"
#include "ADF_U1.h"
#include "ADF_U1_internals.h"



/***********************************************************************
    ADF Utility Level 1 subroutines:  Implementation and testing status.
:'x,'z ! sort -f
X = done  s=started  w=working (more to add)
   status of routines:             | .h  | .c  | .c  | .c  | .c  |
   status of routines:         rel |proto| stub| code|testd| errs|
                                   |  -  |  -  |  -  |  -  |  -  |
                                   +-----+-----+-----+-----+-----+
AU1_Count_Nodes                 1  |  X  |  X  |  X  |  -  |  -  |
AU1_Return_Node_IDs             1  |  X  |  X  |  X  |  -  |  -  |
AU1_Count_Ancestors             1  |  X  |  X  |  X  |  -  |  -  |
AU1_Return_Ancestor_IDs         1  |  X  |  X  |  X  |  -  |  -  |


                                   +-----+-----+-----+-----+-----+
***********************************************************************/



#ifndef FALSE
#define FALSE    0
#endif
#ifndef TRUE
#define TRUE     1
#endif

typedef int Boolean;

/***********************************************************************
    Error strings
    These strings must be kept in sync with the error defines in 
    ADF_U1_internals.h.
***********************************************************************/
const char  *AU1_error_string[] = {
   "AU1 -1: No Error.",
   "AU1  1: An error occurred in an ADF core routine.",
   "AU1  2: A loop in the node graph was detected.",
   "AU1  3: A string pointer is NULL.",
   "AU1  4: String length longer than maximum allowable length.",
   "AU1  5: Integer number is less than given minimum value.",
   "AU1  6: A pointer is NULL.",


/*

   "AU1  4: String length longer than maximum allowable length.",
   "AU1  5: String is not an ASCII-HEX string.",
   "AU1  6: Too many ADF files opened.",
   "AU1  7: ADF file status was not recognized.",
   "AU1  8: ADF file-open error.",
   "AU1  9: ADF file not currently opened.",
   "AU1 10: ADF file index out of legal range.",
   "AU1 11: Block/offset out of legal range.",
   "AU1 12: A string pointer is NULL.",
   "AU1 13: FSEEK error.",
   "AU1 14: FWRITE error.",
   "AU1 15: FREAD error.",
   "AU1 16: Internal error: Memory boundary tag bad.",
   "AU1 17: Internal error: Disk boundary tag bad.",
   "AU1 18: File Open Error: NEW - File already exists.",
   "AU1 19: ADF file format was not recognized.",
   "AU1 20: Attempt to free the RootNode disk information.",
   "AU1 21: Attempt to free the FreeChunkTable disk information.",
   "AU1 22: File Open Error: OLD - File does not exist.",
   "AU1 23: Entered area of Unimplemented Code...",
   "AU1 24: Sub-Node.entries is bad.",
   "AU1 25: Memory allocation failed.",
   "AU1 26: Duplicate child name under a parent node.",
   "AU1 27: Node as no dimensions.",
   "AU1 28: Node's number-of-dimensions is not in legal range.",
   "AU1 29: Specified child is NOT a child of the specified parent.",
   "AU1 30: Data-Type is too long.",
   "AU1 31: Invalid Data-Type.",
   "AU1 32: A pointer is NULL.",
   "AU1 33: Node has no data associated with it.",
   "AU1 34: Error zeroing out memory.",
   "AU1 35: Requested data exceeds actual data available.",
   "AU1 36: Bad end value.",
   "AU1 37: Bad stride value.",
   "AU1 38: Minimum values is greater than the maximum value.",
   "AU1 39: The format of this machine does not match a known signature.",
   "AU1 40: Cannot convert to or from an unknown Native format.",
   "AU1 41: The two conversion formats are equal, no conversion done.",
   "AU1 42: The data format is not support on a particular machine.",
   "AU1 43: File Close error.",
   "AU1 44: Numeric overflow/underflow in data conversion.",
   "AU1 45: Bad start value.",
   "AU1 46: A value of zero is not allowable.",
   "AU1 47: Bad dimension value.",
   "AU1 48: Error state must be either a 0 (zero) or a 1 (one).",
   "AU1 49: Dimensional specifications for disk and memory are unequal.",
   "AU1 50: Too many link level used.  May be caused by a recursive link.",
   "AU1 51: The node is not a link.  It was expected to be a link.",
   "AU1 52: The linked-to node does not exist.",
   "AU1 53: The ADF file of a linked-node is not accessable.",
   "AU1 54: A node-id of 0.0 is not valid.",
   "AU1 55: Incomplete Data when reading multiple data blocks.",
   "AU1 56: Node name contains invalid characters.",
   "AU1 57: ADF file version incompatible with this library version.",
   "AU1 58: Nodes are not from the same file.",
   "AU1 59: Priority Stack Error.",
   "AU1 60: Machine format and file format are incompatable.",
   "AU1 61: FFLUSH error",
   "AU1  x: Last error mesage"
*/
   } ;


/***********************************************************************
    Global Variables:
***********************************************************************/
       int  ADF_abort_on_error = FALSE ;
static int  AU1_abort_on_error = FALSE ;

/*  For error handling, we need parallel devices; ie. to check and report
	error return flags from ADF calls, we need to use definitions and 
	routines provided in the core.  For error return flags from AU1
	calls, we use definitions and routines provided here 
*/

extern const char *ADF_error_string[];  /* in ADF_interface.c */

#define CHECK_ADF_ABORT( error_flag ) if( error_flag != NO_ERROR ) { \
                    if( ADF_abort_on_error == TRUE ) {    \
                      ADF_Error_Message( error_flag, 0L );\
                      AU1I_Abort( error_flag) ; }         \
                    else { return ; } }

#define CHECK_AU1_ABORT( error_flag ) if( error_flag != NO_ERROR ) {\
                    if( AU1_abort_on_error == TRUE ) {    \
                      AU1_Error_Message( error_flag, 0L );\
                      AU1I_Abort( error_flag) ; }         \
                    else { return ; } }

static char scratchname[33], scratchlabel[33];
static double *scratchid;


/***********************************************************************
AU1 Count nodes:

Given inputs of a name and a label, count the number of nodes at or below
the current node whose name and label match the given name and label.  An
input of length zero for the name is taken to mean that any node name is
a match.  Similarly for the label input.  (Thus, for example, making this
call at the root node with zero-length inputs for both strings will yield
the total number of nodes in the file.)

The depth argument can be used to limit the depth of the search for 
matches.  Values of depth are interpreted as follows:  
    1)  Any negative value will cause an unlimited search
    2)  A value of zero causes only the current node (PID) to be
        searched; a value of 1 causes the search to extend to the
        children of the current node (PID); a value of 2 extends the
        search to the children of the children, etc. 

A rough check is made for the case where the nodal graph contains a loop.
Such a loop will cause an error return with no other useful information.

Wildcards in the inputs might be useful but are not implemented.


AU1_Count_Nodes( ID, name, label, depth, num_nodes, error_return )
input:  const double PID    The ID of the Node to start at.
input:  const char *name    The Node name to search for.
input:  const char *label   The Node label to search for.
input:  const int   depth   Maximum depth for the search.
output: int *num_nodes      The number of nodes with matching name,label.
output: int *error_return   Error return.

   Possible errors:
NO_ERROR
AU1_NULL_STRING_POINTER
AU1_STRING_LENGTH_TOO_BIG
AU1_GRAPH_LOOP  
***********************************************************************/

void    AU1_Count_Nodes(
            const double ID,
            const char *name,
            const char *label,
            const int depth,
            int *num_nodes,
            int *error_return )
{
  int      max_nodes = 1000000;  /* wrj: do a calculation based on file
                                         size */
  int      current_depth;

  *error_return = NO_ERROR;
  if( label == NULL || name == NULL) 
  {
    *error_return = AU1_NULL_STRING_POINTER ;
    CHECK_AU1_ABORT( *error_return ) ;
  }
  if( strlen( name  ) > ADF_NAME_LENGTH )
  {
    *error_return = AU1_STRING_LENGTH_TOO_BIG ;
    CHECK_AU1_ABORT( *error_return ) ;
  }
  if( strlen( label ) > ADF_NAME_LENGTH )
  {
    *error_return = AU1_STRING_LENGTH_TOO_BIG ;
    CHECK_AU1_ABORT( *error_return ) ;
  }

  *num_nodes = 0;
  current_depth = 0;
  AU1I_check_node_for_matches(ID,name,label,max_nodes,depth,current_depth,num_nodes,error_return);
  CHECK_AU1_ABORT( *error_return ) ;

}

/***********************************************************************
AU1 return node ids:

Given inputs of a node id, a name and a label, return the ids for 
the istart through istart + ilen - 1 nodes at or below the given node 
whose name and label match the given name and label.  An input of length 
zero for the name is taken to mean that any node name is a match.  
Similarly for the label input.  

The interpretation of the depth argument is the same as in the routine
AU1_Count_Nodes.

The order of obtaining the ids is determined by the order in which the
recursive search finds them.  It should be consistent as long as the 
nodal structure and order of children remains constant. 

A rough check is made for the case where the nodal graph contains a loop.
Such a loop will cause an error return with no other useful information.


AU1_Return_Node_IDs( ID, name, label, istart, ilen, depth, ilen_ret, 
                     node_ids, error_return )
input:  const double ID     The ID of the Node to start from.
input:  const char *name    The Node name to search for.
input:  const char *label   The Node label to search for.
input:  const int istart    The ith node's id to start with (first is 1).
input:  const int ilen      Maximum number of ids to return.
input:  const int depth     Maximum depth for the search.
output: int *ilen_ret       The number of ids returned.
output: double *node_id     The returned ids.
output: int *error_return   Error return.

   Possible errors:
NO_ERROR
AU1_NULL_STRING_POINTER
AU1_NULL_POINTER
AU1_STRING_LENGTH_TOO_BIG
AU1_GRAPH_LOOP  
AU1_NUMBER_LESS_THAN_MINIMUM
***********************************************************************/

void    AU1_Return_Node_IDs(
            const double ID,
            const char *name,
            const char *label,
            const int istart,
            const int ilen,
            const int depth,
            int *ilen_ret,
            double *node_id,
            int *error_return )
{
  int      num_nodes;
  int      current_depth;

  *error_return = NO_ERROR;
  if( label == NULL || name == NULL) 
  {
    *error_return = AU1_NULL_STRING_POINTER ;
    CHECK_AU1_ABORT( *error_return ) ;
  }
  if( strlen( name  ) > ADF_NAME_LENGTH )
  {
    *error_return = AU1_STRING_LENGTH_TOO_BIG ;
    CHECK_AU1_ABORT( *error_return ) ;
  }
  if( strlen( label ) > ADF_NAME_LENGTH )
  {
    *error_return = AU1_STRING_LENGTH_TOO_BIG ;
    CHECK_AU1_ABORT( *error_return ) ;
  }
  if( ilen_ret == NULL ) 
  {
    *error_return = AU1_NULL_POINTER ;
    CHECK_AU1_ABORT( *error_return ) ;
  }
  *ilen_ret = 0 ;

  if( node_id == NULL ) 
  {
    *error_return = AU1_NULL_POINTER ;
    CHECK_AU1_ABORT( *error_return ) ;
  }

  if( (istart <=0) || (ilen <= 0) )
  {
    *error_return = AU1_NUMBER_LESS_THAN_MINIMUM ;
    CHECK_AU1_ABORT( *error_return ) ;
  }

  num_nodes = 0;
  *ilen_ret = 0;
  current_depth = 0;
  AU1I_find_nth_match(ID,name,label,istart,ilen,&num_nodes,depth,current_depth,ilen_ret,node_id,error_return);
  CHECK_AU1_ABORT( *error_return ) ;
}
/***********************************************************************
AU1 count ancestors: Given inputs of a node id, a name and a label, 
        return the number of nodes having the input name and label 
        that have as a child the node with input id.

    An input of length zero (or blank in fortran) for the name is taken 
        to mean that any node name is a match.  Similarly for the label 
        input.  

    If no qualifying ancestor is located, an id of 0 is returned;


C:      AU1_Find_Ancestor( ID, name, label, num_ancestors, error_return )
    input:  const double ID     The ID of the Node.
    input:  const char *name    The Ancestor Node name to search for.
    input:  const char *label   The Ancestor Node label to search for.
    output: int *num_ancestors  The number of ancestors.
    output: int *error_return   Error return.
 
f77:    AU1CAN(id,name,label,nancstr,ierr)
    input:  real*8 id.
    input:  character*(*) name.
    input:  character*(*) label.
    output:  integer nancstr
    output:  integer ierr.

   Possible errors:
NO_ERROR
AU1_NULL_STRING_POINTER
AU1_NULL_POINTER
AU1_STRING_LENGTH_TOO_BIG
AU1_GRAPH_LOOP  
AU1_NUMBER_LESS_THAN_MINIMUM
***********************************************************************/

void    AU1_Count_Ancestors(
            const double ID,
            const char *name,
            const char *label,
            int *num_ancestors,
            int *error_return )
{
  int      num_nodes, max_nodes;
  int      i;
  int      ilen, istart, depth, ilen_ret;
  int      found;
  double   root_ID;
  double   *tmpid;

  *error_return = NO_ERROR;
  if( label == NULL || name == NULL) 
  {
    *error_return = AU1_NULL_STRING_POINTER ;
    CHECK_AU1_ABORT( *error_return ) ;
  }
  if( strlen( name  ) > ADF_NAME_LENGTH )
  {
    *error_return = AU1_STRING_LENGTH_TOO_BIG ;
    CHECK_AU1_ABORT( *error_return ) ;
  }
  if( strlen( label ) > ADF_NAME_LENGTH )
  {
    *error_return = AU1_STRING_LENGTH_TOO_BIG ;
    CHECK_AU1_ABORT( *error_return ) ;
  }

    /* get root id */

  ADF_Get_Root_ID(ID,&root_ID,error_return);
  CHECK_ADF_ABORT( *error_return ) ;

    /* get the ids of all nodes the satisfy the input name and label */

  max_nodes = 100; /* to start with */
  scratchid = (double *)malloc(max_nodes * sizeof(double));
  /* wrj: need a malloc success check */

  num_nodes = 0;
  istart = 1;
  ilen = 100;
  depth = -1;
  while(1)
  {
    AU1_Return_Node_IDs(root_ID, name, label, istart, ilen, depth, 
                                  &ilen_ret,scratchid,error_return);
    if(*error_return != NO_ERROR)
      CHECK_AU1_ABORT( *error_return ) ;

    num_nodes += ilen_ret;

    if(ilen_ret < ilen) /* found all there are */
      break;

    istart += ilen;
    tmpid = scratchid;
    max_nodes += 100;

    scratchid = (double *)realloc((void *)tmpid,
                          (size_t) (max_nodes * sizeof(double))); 
    /* wrj: need a realloc success check */
    
  }

    /* now test each possible ancestor to see if it really is one */

  *num_ancestors = 0;

  for(i=0; i<num_nodes; i++)
  {
    found = 0;
    AU1I_check_kids_for_id_match(scratchid[i],ID,&found,error_return);
    CHECK_AU1_ABORT( *error_return ) ;
    if(found)
      *num_ancestors += 1;
  }

  free(scratchid);
}

/***********************************************************************
AU1 return Ancestor ids: Given inputs of a node id, a name and a label, 
    return the istart through istart + ilen - 1 ids of nodes having 
    the input name and label that have as a child the node with 
    input id.

    An input of length zero (or blank in fortran) for the name is taken 
        to mean that any node name is a match.  Similarly for the label 
        input.  

C:      AU1_Return_Ancestor_IDs(
    input:  const double ID     The ID of the Node.
    input:  const char *name    The Ancestor Node name to search for.
    input:  const char *label   The Ancestor Node label to search for.
    output: const int istart    The ith node's id to start with (first is 1). 
    output: const int ilen      Maximum number of ids requested.
    output: int *ilen_ret       The number of ids returned.
    output: double *node_id     The returned ids.
    output: int *error_return   Error return.

f77:    AU1RAID(id,name,label,istart,ilen,ilen_ret,node_id,ierr)
    input:  real*8 id.
    input:  character*(*) name.
    input:  character*(*) label.
    input:  integer istart.
    input:  integer ilen.
    output:  integer ilen_ret.
    output:  real*8 node_id.
    output:  integer ierr.

   Possible errors:
NO_ERROR
AU1_NULL_STRING_POINTER
AU1_NULL_POINTER
AU1_STRING_LENGTH_TOO_BIG
AU1_GRAPH_LOOP  
AU1_NUMBER_LESS_THAN_MINIMUM
***********************************************************************/

void    AU1_Return_Ancestor_IDs(
            const double ID,
            const char *name,
            const char *label,
            const int istartin,
            const int ilenin,
            int *ilen_retin,
            double *node_id,
            int *error_return )
{
  int      num_nodes, max_nodes, num_ancestors;
  int      i;
  int      ilen, istart, depth, ilen_ret;
  int      found;
  double   root_ID;
  double   *tmpid;

  *error_return = NO_ERROR;
  if( label == NULL || name == NULL) 
  {
    *error_return = AU1_NULL_STRING_POINTER ;
    CHECK_AU1_ABORT( *error_return ) ;
  }
  if( strlen( name  ) > ADF_NAME_LENGTH )
  {
    *error_return = AU1_STRING_LENGTH_TOO_BIG ;
    CHECK_AU1_ABORT( *error_return ) ;
  }
  if( strlen( label ) > ADF_NAME_LENGTH )
  {
    *error_return = AU1_STRING_LENGTH_TOO_BIG ;
    CHECK_AU1_ABORT( *error_return ) ;
  }

    /* get root id */

  ADF_Get_Root_ID(ID,&root_ID,error_return);
  CHECK_ADF_ABORT( *error_return ) ;

    /* get the ids of all nodes the satisfy the input name and label */

  max_nodes = 100; /* to start with */
  scratchid = (double *)malloc(max_nodes * sizeof(double));
  /* wrj: need a malloc success check */

  num_nodes = 0;
  istart = 1;
  ilen = 100;
  depth = -1;
  while(1)
  {
    AU1_Return_Node_IDs(root_ID, name, label, istart, ilen, depth, 
                                  &ilen_ret,scratchid,error_return);
    if(*error_return != NO_ERROR)
      CHECK_AU1_ABORT( *error_return ) ;

    num_nodes += ilen_ret;

    if(ilen_ret < ilen) /* found all there are */
      break;

    istart += ilen;
    tmpid = scratchid;
    max_nodes += 100;

    scratchid = (double *)realloc((void *)tmpid,
                          (size_t) (max_nodes * sizeof(double))); 
    /* wrj: need a realloc success check */
    
  }

    /* now test each possible ancestor to find the "ith" one */

  num_ancestors = 0;
  *ilen_retin = 0;

  for(i=0; i<num_nodes; i++)
  {
    found = 0;
    AU1I_check_kids_for_id_match(scratchid[i],ID,&found,error_return);
    CHECK_AU1_ABORT( *error_return ) ;
    if(found)
    {
      num_ancestors += 1;

      if(num_ancestors >= ilenin)
      {
        node_id[num_ancestors - ilenin] = scratchid[i];
        *ilen_retin += 1;
      }
      
      if(num_ancestors >= (istartin + ilenin - 1))
        break;
    }
  }

  free(scratchid);
}
/* file AU1_Error_Message.c */
/***********************************************************************
AU1 Error message:

Return Error Message.  Given an error_return from an AU1 routine, 
get a textual description of the error.  This is the same as the 
routine ADF_Error_Message in the ADF core except here we get the
error string from the array AU1_error_string.

AU1_Error_Message( error_return, error_string )
input:  const int error_return  An AU1-generated error code.
output: char *error_string  An 80-byte description of the specified error.
            If the number is NULL, then print out error message.
***********************************************************************/
void    AU1_Error_Message(
        const int error_return_input,
        char *error_string )
{
char  err_msg_str[ADF_MAX_ERROR_STR_LENGTH+1] ;

    /** If return pointer is NULL, print message to stdout **/
if( error_string == NULL ) {
   AU1_Error_Message( error_return_input, err_msg_str ) ;
   printf( "%s\n", err_msg_str ) ;
   return ;
   } /* end if */

    /** NO_ERROR is NOT zero for pointer-assignment checking **/
if( error_return_input == NO_ERROR ) {
   strcpy( error_string, AU1_error_string[ 0 ] ) ;
   } /* end if */
    /** Check range of error code **/
else if( (error_return_input <= 0) ||
     (error_return_input >= sizeof( AU1_error_string )/sizeof(char *) - 1 ) ) {
   sprintf( error_string, "AU1: Unrecognized error number %d.",
                error_return_input ) ;
   } /* end else if */
    /** Error-code good, copy it for the user **/
else {
   strcpy( error_string, AU1_error_string[error_return_input] ) ;
   } /* end else */
} /* end of AU1_Error_Message */
