#include <stdlib.h>
#include <stdio.h>
#include <ctype.h>
#include <string.h>
#include "ADF.h"
#include "ADF_U1.h"
#include "ADF_U1_internals.h"

#ifndef FALSE
#define FALSE    0
#endif
#ifndef TRUE
#define TRUE     1
#endif

typedef int Boolean;

/***********************************************************************
    ADF Utility Level 1 subroutines:  Implementation and testing status.
:'x,'z ! sort -f
X = done  s=started  w=working (more to add)
   status of routines:             | .h  | .c  | .c  | .c  | .c  |
   status of routines:         rel |proto| stub| code|testd| errs|
                                   |  -  |  -  |  -  |  -  |  -  |
                                   +-----+-----+-----+-----+-----+
AU1I_check_node_for_matches      1  |  X  |  X  |  X  |  -  |  -  |
AU1I_find_nth_match              1  |  X  |  X  |  X  |  -  |  -  |
AU1I_check_kids_for_id_match     1  |  X  |  X  |  X  |  -  |  -  |


                                   +-----+-----+-----+-----+-----+
***********************************************************************/

/*  for error handling of ADF error returns  */

extern int  ADF_abort_on_error;

extern const char *ADF_error_string[];

/*  a slight modification of the check_adf_abort from the core  */

#define CHECK_ADF_ABORT( error_flag ) if( error_flag != NO_ERROR ) { \
                    if( ADF_abort_on_error == TRUE ) {    \
                      ADF_Error_Message( error_flag, 0L );\
                      AU1I_Abort( error_flag) ; }         \
                    else { error_flag = ADF_CORE_ERROR_OCCURRED;  return ; } }
/**********************************************************************/



void  AU1I_check_node_for_matches(
            const double ID,
            const char *name,
            const char *label,
            const int max_nodes,
            const int depth,
                  int current_depth,
            int *num_nodes,
            int *error_return )
{
  Boolean  matchname, matchlabel;
  char     ctmp[ ADF_NAME_LENGTH + 1 ];
  int      num_children;
  int      i, ilen, name_length, ilen_ret;
  char     childname[33];
  double   CID;
  

  *error_return = NO_ERROR;

  /* check node */

  matchname = matchlabel = FALSE;

  if(strlen(name)  == 0) matchname  = TRUE;
  if(strlen(label) == 0) matchlabel = TRUE;

  if(matchname == FALSE)
  {
    ADF_Get_Name(ID,ctmp,error_return);
    CHECK_ADF_ABORT( *error_return );

    if(strcmp(ctmp,name) == 0)  matchname  = TRUE;
  }

  if(matchlabel == FALSE)
  {
    ADF_Get_Label(ID,ctmp,error_return);
    CHECK_ADF_ABORT( *error_return );

    if(strcmp(ctmp,label) == 0)  matchlabel  = TRUE;
  }

  if(matchname == TRUE && matchlabel == TRUE) *num_nodes += 1;
  if(*num_nodes > max_nodes)
  {
    *error_return = AU1_GRAPH_LOOP ;
    return;
  }

  /* see if we are out of our depth... */

  current_depth++;
  if(depth >= 0 && current_depth > depth)  return;

  /* now recursively check children of this node */

  ADF_Number_of_Children(ID,&num_children,error_return);
  CHECK_ADF_ABORT( *error_return );

  ilen = 1;
  name_length = 32;
  for(i=1; i<=num_children; i++)
  {
    childname[0] = 0;
    ADF_Children_Names(ID,i,ilen,name_length,&ilen_ret,childname,error_return);
    CHECK_ADF_ABORT( *error_return );

    ADF_Get_Node_ID(ID,childname,&CID,error_return);
    CHECK_ADF_ABORT( *error_return );

    AU1I_check_node_for_matches(CID,name,label,max_nodes,depth,current_depth,num_nodes,error_return);
    if(*error_return != NO_ERROR)  return;
  }
}

void AU1I_find_nth_match(
            const double ID,
            const char *name,
            const char *label,
            const int ibegin,
            const int nids,
                  int *num_nodes,
            const int depth,
                  int current_depth,
            int *nids_ret,
            double *node_id,
            int *error_return )
{
  Boolean  matchname, matchlabel;
  char     ctmp[ ADF_NAME_LENGTH + 1 ];
  int      num_children;
  int      i, ilen, name_length, ilen_ret;
  char     childname[33];
  double   CID;

  *error_return = NO_ERROR;

  /* check node */

  matchname = matchlabel = FALSE;

  if(strlen(name)  == 0) matchname  = TRUE;
  if(strlen(label) == 0) matchlabel = TRUE;

  if(matchname == FALSE)
  {
    ADF_Get_Name(ID,ctmp,error_return);
    CHECK_ADF_ABORT( *error_return );

    if(strcmp(ctmp,name) == 0)  matchname  = TRUE;
  }

  if(matchlabel == FALSE)
  {
    ADF_Get_Label(ID,ctmp,error_return);
    CHECK_ADF_ABORT( *error_return );

    if(strcmp(ctmp,label) == 0)  matchlabel  = TRUE;
  }

  if(matchname == TRUE && matchlabel == TRUE) 
  {
    *num_nodes += 1;
    if( *num_nodes >= ibegin && *num_nodes < (ibegin + nids) )
    {
      node_id[*num_nodes - ibegin] = ID; 
      *nids_ret += 1;
    }
  }

  /* see if we have found the requested number of nodes */

  if(*nids_ret >= nids)  return;

  /* see if we are out of our depth... */

  current_depth++;
  if(depth >= 0 && current_depth > depth)  return;

  /* now recursively check children of this node */

  ADF_Number_of_Children(ID,&num_children,error_return);
  CHECK_ADF_ABORT( *error_return );

  ilen = 1;
  name_length = 32;
  for(i=1; i<=num_children; i++)
  {
    childname[0] = 0;
    ADF_Children_Names(ID,i,ilen,name_length,&ilen_ret,childname,error_return);
    CHECK_ADF_ABORT( *error_return );

    ADF_Get_Node_ID(ID,childname,&CID,error_return);
    CHECK_ADF_ABORT( *error_return );

    AU1I_find_nth_match(CID,name,label,ibegin,nids,num_nodes,depth,current_depth,nids_ret,node_id,error_return);
    if(*error_return != NO_ERROR)  return;

    /* check again to see if we have found the requested number of nodes */

    if(*nids_ret >= nids)  return;
  }
}

void  AU1I_check_kids_for_id_match(
            const double PID,
            const double target_id,
            int *found,
            int *error_return )
{
  int      num_children;
  int      i, ilen, name_length, ilen_ret;
  char     childname[33];
  double   CID;
  

  *error_return = NO_ERROR;

  /* recursively check children of this node for a match to the target_id */

  ADF_Number_of_Children(PID,&num_children,error_return);
  CHECK_ADF_ABORT( *error_return );

  ilen = 1;
  name_length = 32;
  for(i=1; i<=num_children; i++)
  {
    childname[0] = 0;
    ADF_Children_Names(PID,i,ilen,name_length,&ilen_ret,childname,error_return);
    CHECK_ADF_ABORT( *error_return );

    ADF_Get_Node_ID(PID,childname,&CID,error_return);
    CHECK_ADF_ABORT( *error_return );

    if( CID == target_id)
    {
      *found = 1;
      return;
    }

    AU1I_check_kids_for_id_match(CID,target_id,found,error_return);
    if(*error_return != NO_ERROR)  return;

    if(*found == 1)  return;
  }
}
/* file AU1I_Abort.c */
/* This was the routine ADFI_Abort.c copied from ADF_Internals.c since
   it was considered bad form to link directly to core internal routines
*/
/***********************************************************************
AU1I Abort:
    Do any cleanup and then shut the application down.
    
input:  const int error_code    Error which caused the Abort.
output: -none-          Hey, we ain't coming back...
***********************************************************************/
void    AU1I_Abort(
        const int error_code )
{
printf("ADF Aborted:  Exiting\n" ) ;
exit( error_code ) ;
} /* end of AU1I_Abort */
/* end of file AU1I_Abort.c */
/* file AU1I_string_2_C_string.c */
/* This was the routine ADFI_string_2_C_string.c copied from ADF_Internals.c since
   it was considered bad form to link directly to core internal routines
*/
/**********************************************************************
AU1I string to C string:
    Create a C string of the maximum length (+1 for null) which is
    null terminated and has no trailing blanks.

input:  const char *string      Input string.
input:  const int string_length     Length of input string to use.
output: char *c_string          Returned C string.
output: int *error_return       Error return.

   Possible errors:
NO_ERROR
AU1_NULL_STRING_POINTER
**********************************************************************/
void    AU1I_string_2_C_string(
        const char *string,
        const int string_length,
        char *c_string,
        int *error_return )
{
int i, iend ;

if( (string == NULL) || (c_string == NULL) ) {
   *error_return = AU1_NULL_STRING_POINTER ;
   return ;
   } /* end if */

*error_return = NO_ERROR ;

    /** Skip and trailing blanks **/
for( iend=string_length-1; iend>=0; iend-- ) {
   if( string[ iend ] != ' ' ) {
      break ;
      } /* end if */
   } /* end for */

    /** Copy the non-trailing blank portion of the string **/
for( i=0; i<=iend; i++ )
   c_string[i] = string[i] ;

    /** NULL terminate the C string **/
c_string[i] = '\0' ;
} /* end of AU1I_string_2_C_string */
/* end of file AU1I_string_2_C_string.c */

