/**
File:    ADF_U1_internals.h
  ----------------------------------------------------------------------
                        BOEING
  ----------------------------------------------------------------------
        Project: CGNS
        Author: Wayne Jones   655-5162   wrj9493@lippy.ca.boeing.com
        Date: 7/8/96  
    Purpose: Provide declarations for the internal ADF-Utility set #1
             routines.
  ----------------------------------------------------------------------
  ----------------------------------------------------------------------

**/

#ifndef    ADF_U1_INTERNALS_INCLUDE
#define    ADF_U1_INTERNALS_INCLUDE
#include "ADF_U1.h"

/***********************************************************************
    Defines
***********************************************************************/

/***********************************************************************
    Defined Marcos
***********************************************************************/
#define MIN(a,b) (((a)<(b))?(a):(b))

/***********************************************************************
    Character string defining the data tags:
***********************************************************************/

/***********************************************************************
    Structures:
***********************************************************************/

/***********************************************************************
    Prototypes for Internal ADF_U1 Utility Routines
***********************************************************************/
extern void  AU1I_check_node_for_matches(
            const double ID,
            const char *name,
            const char *label,
            const int max_nodes,
            const int depth,
                  int current_depth,
            int *num_nodes,
            int *error_return );

extern void AU1I_find_nth_match(
            const double ID,
            const char *name,
            const char *label,
            const int istart,
            const int ilen,
                  int *num_nodes,
            const int depth,
                  int current_depth,
            int *ilen_ret,
            double *node_id,
            int *error_return );

extern void  AU1I_check_kids_for_id_match(
            const double PID,
            const double target_id,
            int *found,
            int *error_return );
  
extern void  AU1I_Abort(
            const int error_code );

/***********************************************************************
    Prototypes for the FORTRAN to C Interface Routines
***********************************************************************/
extern void    AU1CN2(
                const double *rid,
                const char *name,
                const int *name_length,
                const char *label,
                const int *label_length,
                const int *depth,
                int *num_nodes,
                int *error_return );

extern void    AU1RNID2(
                const double *rid,
                const char *name,
                const int *name_length,
                const char *label,
                const int *label_length,
                const int *istart,
                const int *ilen,
                const int *depth,
                int *ilen_ret,
                double *node_id,
                int *error_return );

extern void    AU1CAN2(
                const double *rid,
                const char *name,
                const int *name_length,
                const char *label,
                const int *label_length,
                int *num_nodes,
                int *error_return );


extern void    AU1RAID2(
                const double *rid,
                const char *name,
                const int *name_length,
                const char *label,
                const int *label_length,
                const int *istart,
                const int *ilen,
                int *ilen_ret,
                double *node_id,
                int *error_return );




/***********************************************************************
   Error-return values
   These values need to be kept in sync with the error strings in
   file AU1_interface.c

   These are AU1 error codes NOT ADF error codes !!
***********************************************************************/
    /** Don't use zero since you can assign zero to a pointer **/

#define NO_ERROR                       -1
#define ADF_CORE_ERROR_OCCURRED         1
#define AU1_GRAPH_LOOP                  2
#define AU1_NULL_STRING_POINTER         3
#define AU1_STRING_LENGTH_TOO_BIG       4
#define AU1_NUMBER_LESS_THAN_MINIMUM    5
#define AU1_NULL_POINTER                6

#endif
